electrodes = ["Poly_1","Poly_2","Poly_3","Poly_4","Poly_5","Poly_6","Poly_7","M3_1","M3_2","M3_3","M3_4",]
# NGSpy 6.2 file created by SEQUOIA Cell Designer V21.2 (Build 2021/03/22) on Monday, 03 May, 2021
import time
from netgen.csg import *
from ngsolve import *
t0 = time.perf_counter()
#mesh = Mesh("mesh.vol.gz")
geo = CSGeometry("export.geo")
ngmesh = geo.GenerateMesh()
mesh = Mesh(ngmesh)
print(mesh.GetBoundaries())
print(mesh.GetMaterials())
fout = open("capmat.txt", "w")
fout.write(f"# Capacitance Matrix generated by SEQUOIA Cell Designer V21.2 (Build 2021/03/22) on Monday, 03 May, 2021\n")
#fout.write(f"# electrodes = Poly_1|Poly_2|Poly_3|Poly_4|Poly_5|Poly_6|Poly_7|M3_1|M3_2|M3_3|M3_4|\n")
# H1-conforming finite element space
fes = Periodic(H1(mesh, order=3, dirichlet="Poly_1|Poly_2|Poly_3|Poly_4|Poly_5|Poly_6|Poly_7|M3_1|M3_2|M3_3|M3_4|"))
# define trial- and test-functions
u = fes.TrialFunction()
v = fes.TestFunction()
# the right hand side
f = LinearForm(fes)
f += 0 * v * dx
# the bilinear-form
a = BilinearForm(fes, symmetric=True)
a += grad(u)*grad(v)*dx
#c = Preconditioner(a,"multigrid")
c = Preconditioner(a,"local")
#c = Preconditioner(a,"direct")
a.Assemble()
f.Assemble()
t1 = time.perf_counter()
print(f"Assemble {t1 - t0:0.4f} seconds")
# the solution field
gfu = GridFunction(fes)
normals = -CoefficientFunction (specialcf.normal(3))
#bfv2 = BoundaryFromVolumeCF(func_domain2)
#Draw (bfv2, mesh, "normal-vec")
ntot = 11
#ntot = 1
for elec in range(0,ntot):
     t1 = time.perf_counter()
     print("elec=", electrodes[elec], elec)
     gfu.Set(1.0, definedon=mesh.Boundaries(electrodes[elec]))
#     gfu.Set(1.0, definedon=mesh.Materials(str(elec+1)))
     BVP(bf=a, gf=gfu, lf=f, pre=c, maxsteps=100).Do()
     Draw(gfu, mesh, "u")
     t2 = time.perf_counter()
     print(f"BVP{elec} {t2 - t1:0.4f} seconds")
     print("starting gfE.Set( -grad(gfu))")
     fes_flux = HDiv(mesh, order=1)
     gfE = GridFunction(fes_flux)
     gfE.Set( -grad(gfu))
     Draw(gfE, mesh, "E")
     fout.write(f"{electrodes[elec]} ")
     sum=0
     for i in range(0, ntot):
         charge = (Integrate(sqrt((gfE*normals)**2), mesh, definedon=mesh.Boundaries(electrodes[i])))
         if elec != i :
             charge = -charge
         print ("Integral = ", charge)
         sum += charge
         fout.write(f"{charge:0.4f} ")
     fout.write(f"  sum={sum:0.4f}\n")
     print("sum=",sum)
     t2 = time.perf_counter()
     print(f"elec{elec} {t2 - t1:0.4f} seconds")
t3 = time.perf_counter()
fout.write(f"# run time {t3 - t0:0.4f} seconds")
print(f"total {t3 - t0:0.4f} seconds")
fout.close()
#quit()

